/*
 * @Author: Eward 
 * @Date: 2017-08-17 21:24:30 
 * @Last Modified by: Eward
 * @Last Modified time: 2017-09-29 10:24:00
 */
"use strict";

const fs = require("fs"); //eward
const autoprefixer = require("autoprefixer");
const path = require("path");
const webpack = require("webpack");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const CaseSensitivePathsPlugin = require("case-sensitive-paths-webpack-plugin");
const InterpolateHtmlPlugin = require("react-dev-utils/InterpolateHtmlPlugin");
const WatchMissingNodeModulesPlugin = require("react-dev-utils/WatchMissingNodeModulesPlugin");
const eslintFormatter = require("react-dev-utils/eslintFormatter");
const ModuleScopePlugin = require("react-dev-utils/ModuleScopePlugin");
const getClientEnvironment = require("./env");
const paths = require("./paths");
const os = require("os");
const HappyPack = require("happypack");
const happyThreadPool = HappyPack.ThreadPool({ size: os.cpus().length });
const publicPath = "/";
const publicUrl = "";
const env = getClientEnvironment(publicUrl);

const alias =
  (fs.existsSync(paths.appConfig) && require(paths.appConfig).alias) || {};

for (let i in alias) {
  alias[i] = paths.appSrc + alias[i];
}

module.exports = {
  cache: true, //开发环境开启缓存
  devtool: "cheap-module-source-map",
  entry: [
    require.resolve("./polyfills"),
    require.resolve("react-dev-utils/webpackHotDevClient"),
    paths.appIndexJs
  ],
  output: {
    path: paths.appBuild,
    pathinfo: true,
    filename: "static/js/bundle.js",
    chunkFilename: "static/js/[name].chunk.js",
    publicPath: publicPath,
    devtoolModuleFilenameTemplate: info =>
      path.resolve(info.absoluteResourcePath).replace(/\\/g, "/")
  },
  resolve: {
    modules: ["node_modules", paths.appNodeModules].concat(
      process.env.NODE_PATH.split(path.delimiter).filter(Boolean)
    ),
    extensions: [".web.js", ".js", ".json", ".web.jsx", ".jsx"],
    alias: Object.assign(
      //eward
      {},
      {
        "babel-runtime": path.dirname(
          require.resolve("babel-runtime/package.json")
        ),
        "react-native": "react-native-web"
      },
      alias
    ),
    plugins: [new ModuleScopePlugin(paths.appSrc, [paths.appPackageJson])]
  },
  module: {
    strictExportPresence: true,
    rules: [
      {
        test: /\.(js|jsx)$/,
        enforce: "pre",
        use: [
          {
            options: {
              formatter: eslintFormatter,
              eslintPath: require.resolve("eslint"),
              cache:true
            },
            loader: require.resolve("eslint-loader")
          }
        ],
        include: paths.appSrc
      },
      {
        oneOf: [
          {
            test: [/\.bmp$/, /\.gif$/, /\.jpe?g$/, /\.png$/],
            loader: require.resolve("url-loader"),
            options: {
              limit: 10000,
              name: "static/media/[name].[hash:8].[ext]"
            }
          },
          {
            test: /\.(js|jsx)$/,
            include: paths.appSrc,
            loaders: ["happypack/loader"]
          },
          // {
          //   test: /\.(js|jsx)$/,
          //   include: paths.appSrc,
          //   loader: require.resolve("babel-loader"),
          //   options: Object.assign(
          //     {},
          //     (fs.existsSync(paths.appBabelRc) &&
          //       JSON.parse(fs.readFileSync(paths.appBabelRc).toString())) ||
          //       {},
          //     {
          //       presets: (fs.existsSync(paths.appBabelRc) &&
          //         JSON.parse(fs.readFileSync(paths.appBabelRc).toString())
          //           .presets &&
          //         ["react-app"].concat(
          //           JSON.parse(fs.readFileSync(paths.appBabelRc).toString())
          //             .presets
          //         )) || ["react-app"],
          //       cacheDirectory: true
          //     }
          //   )
          // },
          {
            test: /\.css$/,
            use: [
              require.resolve("style-loader"),
              {
                loader: require.resolve("css-loader"),
                options: {
                  importLoaders: 1
                }
              },
              {
                loader: require.resolve("postcss-loader"),
                options: {
                  ident: "postcss",
                  plugins: () => [
                    require("postcss-flexbugs-fixes"),
                    autoprefixer({
                      browsers: [
                        ">1%",
                        "last 4 versions",
                        "Firefox ESR",
                        "not ie < 9" // React doesn't support IE8 anyway
                      ],
                      flexbox: "no-2009"
                    })
                  ]
                }
              }
            ]
          },
          //新增对module less的支持
          {
            //eward
            test: /\.module\.less$/,
            use: [
              require.resolve("style-loader"),
              {
                loader: require.resolve("css-loader"),
                options: {
                  importLoaders: 1,
                  modules: true,
                  sourceMap: false,
                  localIdentName: "[local]___[hash:base64:5]"
                }
              },
              {
                loader: require.resolve("postcss-loader"),
                options: {
                  ident: "postcss",
                  plugins: () => [
                    require("postcss-flexbugs-fixes"),
                    autoprefixer({
                      browsers: [
                        ">1%",
                        "last 4 versions",
                        "Firefox ESR",
                        "not ie < 9" // React doesn't support IE8 anyway
                      ],
                      flexbox: "no-2009"
                    })
                  ]
                }
              },
              {
                loader: require.resolve("less-loader"),
                options: {
                  modifyVars:
                    (fs.existsSync(paths.appConfig) &&
                      require(paths.appConfig).modifyVars) ||
                    {}
                }
              }
            ]
          },
          //新增对less的支持
          {
            //eward
            test: /\.less$/,
            use: [
              require.resolve("style-loader"),
              {
                loader: require.resolve("css-loader")
              },
              {
                loader: require.resolve("postcss-loader"),
                options: {
                  ident: "postcss",
                  plugins: () => [
                    require("postcss-flexbugs-fixes"),
                    autoprefixer({
                      browsers: [
                        ">1%",
                        "last 4 versions",
                        "Firefox ESR",
                        "not ie < 9" // React doesn't support IE8 anyway
                      ],
                      flexbox: "no-2009"
                    })
                  ]
                }
              },
              {
                loader: require.resolve("less-loader"),
                options: {
                  modifyVars:
                    (fs.existsSync(paths.appConfig) &&
                      require(paths.appConfig).modifyVars) ||
                    {}
                }
              }
            ]
          },
          {
            exclude: [/\.js$/, /\.html$/, /\.json$/],
            loader: require.resolve("file-loader"),
            options: {
              name: "static/media/[name].[hash:8].[ext]"
            }
          }
        ].concat(
          (fs.existsSync(paths.appConfig) &&
            require(paths.appConfig).loaders) ||
            []
        ) //eward
      }
    ]
  },
  plugins: [
    //增加happypack进行多进程打包
    new HappyPack({
      threadPool: happyThreadPool,
      cache: true,
      verbose: false,
      loaders: [
        {
          loader: require.resolve("babel-loader"),
          options: Object.assign(
            {},
            (fs.existsSync(paths.appBabelRc) &&
              JSON.parse(fs.readFileSync(paths.appBabelRc).toString())) ||
              {},
            {
              presets: (fs.existsSync(paths.appBabelRc) &&
                JSON.parse(fs.readFileSync(paths.appBabelRc).toString())
                  .presets &&
                [require.resolve("babel-preset-react-app")].concat(
                  JSON.parse(fs.readFileSync(paths.appBabelRc).toString())
                    .presets
                )) || [require.resolve("babel-preset-react-app")],
              cacheDirectory: true
            }
          )
        }
      ]
    }),
    new InterpolateHtmlPlugin(env.raw),
    new HtmlWebpackPlugin({
      inject: true,
      template: paths.appHtml
    }),
    new webpack.NamedModulesPlugin(),
    new webpack.DefinePlugin(env.stringified),
    new webpack.HotModuleReplacementPlugin(),
    new CaseSensitivePathsPlugin(),
    new WatchMissingNodeModulesPlugin(paths.appNodeModules),
    new webpack.IgnorePlugin(/^\.\/locale$/, /moment$/)
  ].concat(
    (fs.existsSync(paths.appConfig) && require(paths.appConfig).devPlugins) ||
      []
  ),
  node: {
    dgram: "empty",
    fs: "empty",
    net: "empty",
    tls: "empty",
    child_process: "empty"
  },
  performance: {
    hints: false
  }
};
